{{******************************************************
* (C) 2009 myDanceBot.com, Inc.                        *
* Conduit v4.3 11/09                                   *
* Share Propeller Memory with ViewPort Application     *
********************************************************
Use this Object to share memory with a ViewPort Application.
Once memory is shared, the value of a variable can be viewed and changed while the
program is running.  Data from other objects can be displayed and their behavior
controlled from ViewPort's graphical interface.
_______________________
Features:
*Transfers data at up to 2Mbps Full-Duplex over a Parallax USB-Serial connection or
 115kbps over other Serial connections.
*Manages configuration of variables and registration of ViewPort objects
*Automatically switches between 2Mbps, 1Mbps and 115kbps depending on ViewPort setting
_______________________
Use:
-Set clock rate to 80MHz:
        CON
         _clkmode        = xtal1 + pll16x
         _xinfreq        = 5_000_000
-Include this object in your program:
        Obj vp:"Conduit"
-Use these ViewPort Functions
  Share(@varA,@varC)            Share memory from varA to varC with ViewPort. This lets you
                                monitor&change them.  You MUST use this function- at the start of your
                                program, but AFTER "register" and "config" statements have run.
  Register(component,cfg)       Register other components (fuzzy logic, video, quicksample..)
  Config(cfgstring)             Configure ViewPort.  Can be generated by ViewPort.
}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  TXPIN=|<30  
  RXPIN=|<31
  PS=True
  DBGLEN=6
PUB debug(dP1,dP2)
{{
 Used by ViewPort debugger
}}
  array(dP1,6,4)
  dbPtr:=dP2  
pub waitpacket|lastpacket
  lastpacket:=packetn
  repeat while packetn==lastpacket
PUB share(firstVarPtr,lastVarPtr)|arrayLen
{{
Share memory from varA to varC with ViewPort. This lets you
monitor&change them.  You MUST use this function- at the start of your
program, but AFTER "register" and "config" statements have run.  
}}
  if lastVarPtr
    array(firstVarPtr,(lastVarPtr-firstVarPtr+4)/4,1)
  resyncP:=(1200/4/totalLen) #>1  
  ''calculate timings
  bittick10:=clkfreq/2_000_000            
  bittick15:=bittick10*3/2               
  bitticks115k:=clkfreq/115_200
  packetNPtr:= @packetN
  cognew(@entry, 0)
  return @packetN
pub array(arrayPtr,arrayLen,arrayType)
  sharePtr[shareNum]:=arrayPtr
  shareLen[shareNum]:=arrayLen
  totalLen+=arrayLen
  shareType+=arrayType<<(shareNum*3+8) 
  shareNum+=1
pub config(sptr)
{{
Configure ViewPort.  CfgString can be generated by ViewPort.
}}
  configlen(sptr,strsize(sptr))
pub configlen(sptr,len)
  long[@txStrPtr0+txStrNum<<2]:=sptr+(len+1)<<24
  txStrNum+=1':=2'+=1  
'  txStrPtr0:=sptr+(len+1)<<24
'  txStrPtr1:=sptr+(len+1)<<24  
  
pub register(mPtr)|mode,cfg,len
{{
Register other components (fuzzy logic, video, quicksample..)
}}
   
   
   
   
  mode:=mptr & 3
  mPtr-=mode
  cfg:=long[mPtr]
  case mode
    0:    
      case cfg
        1..2:len:=6000
        3:len:=6000+1
        4:len:=3000+1
        OTHER: len:=1500
    2: 
   


      qscog:=cfg               
      tcog:=long[mPtr+8]
      ncog:=long[mPtr+12]+1    
      long[mPtr+24]:=@packetn   
      sync:=mptr+28
      if ncog==5
        ncog-=1
        len:=4*258+9     
        psadd2+=1        
      else


      
        psinitT:=0     
        len:=ncog*360+6                    
    3:
   
      len:=cfg/4
  array(mPtr,len,mode)
DAT
{************************************
* Assembly language conduit driver  *
*************************************}

                        org
entry
                        sub     tximask,#1
                        sub     tximask,txmask
                        mov     txcnt,hsec
                        add     txcnt,cnt
                       
                        waitcnt txcnt,#100
                        or      dira,txmask
                        or      outa,txmask
                        jmp     #txcfgstr

receive                 or      outa,txmask
                        mov     txcnt,cnt 
                        waitpeq rxmask,rxmask
                        mov     t1,cnt
                        sub     txcnt,t1
                        mov     t4,#2 
                        mov     t5,#0 
                        neg     txcnt,txcnt 
                        cmp     txcnt,stopticks wc 
            if_c        jmp     #txErr
                        waitpne rxmask,rxmask

                        mov     rxcnt,cnt
                        mov     t3,rxcnt't2-t3
                        sub     t3,t1  
                       
{t2 measures stopbit length.
  if t2<80, 80mhz/2mbit=40cycles
  stopbits is 2x bitticks
  if you're running at a different frequency, change bitticks, stopticks and xtf
  for example, if you're running at 96mhz, bitticks should be=48, stopticks=96}
                        cmp     t3,bittick15 wc    
            if_c        mov     bitticks,bittick10
            if_c        jmp     #bit0
                        mov     t1,bittick15
                        shl     t1,#1 
                        cmp     t3,t1 wc    
            if_c        mov     bitticks,bittick10
            if_c        shl     bitticks,#1 
            if_c        jmp     #bit0                         
                        mov     bitticks,bitticks115k
                        mov     t1,bitticks
                        shr     t1,#1
                        add     rxcnt,t1
bit0
                        mov     stopticks,bitticks
                        shl     stopticks,#1
                        add     rxcnt,bitticks
                        add     rxcnt,#12
bit                    
                        mov     t2,#8
                       
                       
  :bit8                 waitcnt rxcnt,bitticks
                        test    rxmask,ina      wc
                        rcr     t5,#1    
                        djnz    t2,#:bit8
                        waitcnt rxcnt,stopticks
                        djnz    t4,#bit
                        shr     t5,#16   
                        mov     t4,#4
 rxlong
                        mov     t2,#8
                       
  :bit8                 waitcnt rxcnt,bitticks
                        test    rxmask,ina      wc
                        rcr     t1,#1    
                        djnz    t2,#:bit8
                        waitcnt rxcnt,stopticks
                        djnz    t4,#rxlong    
                        cmp     t5,#0 wz
              if_z      add     t1,#:docontrol
              if_z      jmp     t1
                       
                        wrlong  t1,t5   
                        jmp     #txData

:docontrol              jmp     #txErr   
                        nop   
                        jmp     #txData  
                        jmp     #restart 
                        jmp     #txcfgStr
                        jmp     #:leavebkpt'5: leave bkpt
                        jmp     #txAll    
:leavebkpt
                        mov     t2,dbPtr
                        add     t2,#4     
                        rdlong  t3,t2     
                        mov     t1,#0
                        wrlong  t1,t2     
                        mov     rxcnt,cnt
                        add     rxcnt,spinticks
                        waitcnt rxcnt,bitticks 
                        wrlong  t3,t2     
                        wrlong  t1,dbPtr  
                        jmp     #txData
     getTxPtr           mov     t3,sharePtr
     getTxLen           mov     t2,shareLen     
  restart               mov     t1,cnt
                        wrlong  t1,sync
                        mov     t1,qscog   
                        mov     t2,ncog
  startC                coginit t1 nr
   psadd2               add     t1,#1   
                        djnz    t2,#startC
   psinitT              coginit tcog    
txData                  
                        or      shareType,#1
                        mov     txcnt,cnt
                        add     txcnt,stopticks
                        mov     rxcnt,#1
                        call    #txhdr
txD                     mov     DivCnt,#256
                        mov     resynccnt,resyncP                        
txMultB
       
                        mov     doincStatus,#1
       
                        mov     dogetTxPtr,getTxPtr
                        mov     dogetTxLen,getTxLen
                        mov     t4,shareNum
       
                           
      dogetTxPtr        nop
        dogetTxLen      nop
       
  txlongs               call    #transmit
                        djnz    divCnt,#notEOD
             
                        mov     txByte,#26
                        call    #transmit  
                        mov     divCnt,#255
             
      notEOD            rdlong  txByte,t3
                        call    #transmit
                        cmp     txByte,s1 wz
                        add     t3,#4                                
                        call    #transmit
             
                        tjz     doincStatus,#noincStatus            
                        add     packetN,#1
                        wrlong  packetN,packetNPtr
                        mov     doincStatus,#0
  noincStatus           call    #transmit
              if_z      call    #translong
             
              if_z      add     txByte,#6                        
                        djnz    t2,#txlongs
       
                        add     dogetTxPtr,#1
                        add     dogetTxLen,#1
                        djnz    t4,#dogetTxPtr
txDataCon
       
                       
                       
                        djnz    resynccnt,#txMultB
                        call    #transmit
                                                 
        
                        mov     rxcnt,#199
                        neg     rxcnt,rxcnt
                        call    #txhdr   
                        jmp     #txD

       
                      
txErr                   mov     txcnt,cnt
                        add     txcnt,stopticks
                        mov     rxcnt,#5
                        call    #txhdr
                        jmp     #txcfgstr
                        
txhdr                   mov     txByte,s1
                        call    #translong
                        add     txByte,rxcnt
                        ror     txByte,#8
                        call    #translong
txhdr_ret               ret
'sends out memory, header
txAll                   mov     t3,#0
                        mov     rxcnt,#8                        
                        mov     txcnt,cnt
                        add     txcnt,stopticks
                        call    #txhdr
                        call    #transmit
                        mov     t4,#32                     
    :txloop            
                       
                       
                       
                        mov     t2,#256


     :txlongs           call    #transmit
                        rdlong  txByte,t3
                        call    #transmit
                        cmp     txByte,s1 wz
                        add     t3,#4                                
                        call    #transmit
                        call    #transmit
              if_z      call    #translong
              if_z      add     txByte,#6                        
                        djnz    t2,#:txlongs
                        



                        mov     txByte,#26
                        call    #transmit
                        djnz    t4,#:txloop

                        mov     rxcnt,#7
                        call    #txhdr
                        jmp     #txdata                                                                       

txCfgStr                mov     txcnt,cnt
                        add     txcnt,stopticks                        
    :txloop
                        mov     txByte,shareType
                        call    #transLong
                        mov     :getHdrPtr,getHdrPtr
                        mov     :getHdrLen,getHdrLen
                        mov     :getStr,getStrMov
                        mov     t2,#8        
        :getHdrPtr      nop
                        call    #transLong
        :getHdrLen      nop
                        call    #transLong          
                        add     :getHdrPtr,#1
                        add     :getHdrLen,#1
                        djnz    t2,#:getHdrPtr
                        mov     txByte,dbptr           
                        call    #translong  
                        rdlong  txByte,#0              
                        call    #translong
                        mov     t4,txStrNum
       :getStr          nop
                        add     :getStr,#1
                        call    #txStr
                 if_nz  djnz    t4,#:getStr        
                        mov     rxcnt,#4
                        call    #txhdr
                        jmp     #:txloop                        
   getHdrPtr            mov     txByte,sharePtr
   getHdrLen            mov     txByte,shareLen
   getStrMov            mov     t2,txstrptr0  
   low24                long    $00FFFFFF
txStr                   add     txcnt,bitticks
                        mov     t3,t2      
                        shr     t3,#24 wz
                        and     t2,low24   
                        mov     txbyte,t3
    :loop      if_nz    call    #transmit  
               if_nz    rdbyte  txbyte,t2
                        add     t2,#1
               if_nz    djnz    t3,:loop
txStr_ret               ret                        

transLong
                        call    #transmit
                        call    #transmit
                        call    #transmit
                        call    #transmit
transLong_ret           ret

transmit                waitcnt txcnt,bitticks   
                        and     outa,tximask    
                        mov     t1,#8
:bit                    test    rxmask,ina      wc                        
            if_nc       jmp     #receive
                        ror     txByte,#1       wc
                        waitcnt txcnt,bitticks     
                        muxc    outa,txmask
                        djnz    t1,#:bit
                        waitcnt txcnt,stopticks                                          
                        or      outa,txmask     
transmit_ret            ret
sharePtr                long    0,1,2,3,4,5,6,7
shareLen                long    0,0,0,0,0,0,0,0
shareType               long    4
shareNum                long    0                
totalLen                long    0
bitticks                long    40
stopticks               long    80
s1                      long    $E1E1E1E1
txstrnum                long    0
txstrptr0               long    0
txstrptr1               long    0
txstrptr2               long    0
txstrptr3               long    0
txstrptr4               long    0
txstrptr5               long    0
hsec                    long    8_000_000
resyncp                 long    0 
resynccnt               long    0 


qscog                   long    0
tcog                    long    0
sync                    long    0
ncog                    long    0

t1                      long    0 
t2                      long    0
t3                      long    0
t4                      long    0
t5                      long    0
rxmask                  long    RXPIN
txmask                  long    TXPIN
tximask                 long    0
spinticks               long    2000
dbptr                   long    1
{constants for slow serial connection- at 80mhz equates to 115kbps.
xt is the time in cycles to send 360 longs, at 96mhz should be 17_280_000}
bittick10               long    0
bittick15               long    0
bitticks115k            long    0
packetN                 long    0
packetNptr              long    0
xt                      res     1
rxcnt                   res     1
txByte                  res     1
txcnt                   res     1
doIncStatus             res     1
divcnt                  res     1
